//
//  ContentMainTabContentViewModel.swift
//  SodaLive
//
//  Created by klaus on 2/21/25.
//

import Foundation
import Combine

final class ContentMainTabContentViewModel: ObservableObject {
    private let repository = ContentMainTabContentRepository()
    private var subscription = Set<AnyCancellable>()
    
    @Published var errorMessage = ""
    @Published var isShowPopup = false
    @Published var isLoading = false
    
    @Published var bannerList: [GetAudioContentBannerResponse] = []
    @Published var contentThemeList: [String] = []
    @Published var newContentList: [GetAudioContentMainItem] = []
    @Published var rankSortTypeList: [String] = []
    @Published var rankContentList: [GetAudioContentRankingItem] = []
    @Published var contentRankCreatorList: [ContentCreatorResponse] = []
    @Published var likeCountRankContentList: [GetAudioContentRankingItem] = []
    @Published var eventBannerList: [EventItem] = []
    @Published var tagList: [String] = []
    @Published var tagCurationContentList: [GetAudioContentMainItem] = []
    @Published var curationList: [GetContentCurationResponse] = []
    
    func fetchData() {
        isLoading = true
        repository.getContentMainContent()
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<GetContentMainTabContentResponse>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.bannerList = data.bannerList
                        self.contentThemeList = ["전체"] + data.contentThemeList
                        self.newContentList = data.newContentList
                        self.rankSortTypeList = data.rankSortTypeList
                        self.rankContentList = data.rankContentList
                        self.contentRankCreatorList = data.contentRankCreatorList
                        self.likeCountRankContentList = data.likeCountRankContentList
                        self.eventBannerList = data.eventBannerList.eventList
                        self.tagList = data.tagList
                        self.tagCurationContentList = data.tagCurationContentList
                        self.curationList = data.curationList
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
    
    func getNewContentOfTheme(theme: String) {
        isLoading = true
        
        repository.getNewContentOfTheme(theme: theme == "전체" ? "" : theme)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<[GetAudioContentMainItem]>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.newContentList = data
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
    
    func getContentRanking(sort: String = "매출") {
        isLoading = true
        repository.getContentRanking(sortType: sort)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<[GetAudioContentRankingItem]>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.rankContentList = data
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
    
    func getRecommendContentByTag(tag: String) {
        isLoading = true
        repository.getRecommendContentByTag(tag: tag)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<[GetAudioContentMainItem]>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.tagCurationContentList = data
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
    
    func getPopularContentByCreator(creatorId: Int) {
        isLoading = true
        repository.getPopularContentByCreator(creatorId: creatorId)
            .sink { result in
                switch result {
                case .finished:
                    DEBUG_LOG("finish")
                case .failure(let error):
                    ERROR_LOG(error.localizedDescription)
                }
            } receiveValue: { [unowned self] response in
                let responseData = response.data
                
                do {
                    let jsonDecoder = JSONDecoder()
                    let decoded = try jsonDecoder.decode(ApiResponse<[GetAudioContentRankingItem]>.self, from: responseData)
                    
                    if let data = decoded.data, decoded.success {
                        self.likeCountRankContentList = data
                    } else {
                        if let message = decoded.message {
                            self.errorMessage = message
                        } else {
                            self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                        }
                        
                        self.isShowPopup = true
                    }
                } catch {
                    self.errorMessage = "다시 시도해 주세요.\n계속 같은 문제가 발생할 경우 고객센터로 문의 주시기 바랍니다."
                    self.isShowPopup = true
                }
                
                self.isLoading = false
            }
            .store(in: &subscription)
    }
}
